@@ -226,7 +226,8 @@ void _embind_register_enum(
     TYPEID enumType,
     const char* name,
     size_t size,
-    bool isSigned);
+    bool isSigned,
+    int policyValue);
 
 void _embind_register_smart_ptr(
     TYPEID pointerType,
@@ -273,7 +274,7 @@ void _embind_register_bindings(struct InitFunc* f);
 // to avoid static constructor ordering issues.
 struct InitFunc {
   InitFunc(void (*init_func)()) : init_func(init_func) {
-    // This the function immediately upon constructions, and also register
+    // This calls the function immediately upon construction, and also registers
     // it so that it can be called again on each worker that starts.
     init_func();
     _embind_register_bindings(this);
@@ -1702,8 +1703,8 @@ public:
         auto ster = &SP::template set<ClassType>;
 
         typename WithPolicies<Policies...>::template ArgTypeList<typename GP::ReturnType> returnType;
-        // XXX: This currently applies all the polices (including return value polices) to the
-        // setter function argument to allow pointers. Using return value polices doesn't really
+        // XXX: This currently applies all the policies (including return value policies) to the
+        // setter function argument to allow pointers. Using return value policies doesn't really
         // make sense on an argument, but we don't have separate argument policies yet.
         typename WithPolicies<Policies...>::template ArgTypeList<typename SP::ArgumentType> argType;
 
@@ -1918,34 +1919,7 @@ class_<std::map<K, V, Compare, Allocator>> register_map(const char* name) {
         ;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// std::optional
-////////////////////////////////////////////////////////////////////////////////
 
-namespace internal {
-template <typename T>
-struct BindingType<std::optional<T>> {
-    using ValBinding = BindingType<val>;
-    using WireType = ValBinding::WireType;
-
-    template<typename ReturnPolicy = void>
-    static WireType toWireType(std::optional<T> value, rvp::default_tag) {
-        if (value) {
-            return ValBinding::toWireType(val(*value, allow_raw_pointers()), rvp::default_tag{});
-        }
-        return ValBinding::toWireType(val::undefined(), rvp::default_tag{});
-    }
-
-
-    static std::optional<T> fromWireType(WireType value) {
-        val optional = val::take_ownership(value);
-        if (optional.isUndefined()) {
-            return {};
-        }
-        return optional.as<T>();
-    }
-};
-} // end namespace internal
 
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1957,13 +1931,14 @@ class enum_ {
 public:
     typedef EnumType enum_type;
 
-    enum_(const char* name) {
+    enum_(const char* name, enum_value_type valueType = enum_value_type::object) {
         using namespace internal;
         _embind_register_enum(
             internal::TypeID<EnumType>::get(),
             name,
             sizeof(EnumType),
-            std::is_signed<typename std::underlying_type<EnumType>::type>::value);
+            std::is_signed<typename std::underlying_type<EnumType>::type>::value,
+            static_cast<int>(valueType));
     }
 
     enum_& value(const char* name, EnumType value) {
