@@ -22,7 +22,6 @@ import textwrap
 from functools import wraps
 from pathlib import Path
 from subprocess import PIPE, STDOUT
-from typing import Dict, Tuple
 
 import clang_native
 import jsrun
@@ -35,6 +34,7 @@ from tools.settings import COMPILE_TIME_SETTINGS
 from tools.shared import DEBUG, EMCC, EMXX, get_canonical_temp_dir
 from tools.utils import (
   WINDOWS,
+  exe_path_from_root,
   exit_with_error,
   path_from_root,
   read_binary,
@@ -78,13 +78,13 @@ TEST_ROOT = path_from_root('test')
 LAST_TEST = path_from_root('out/last_test.txt')
 PREVIOUS_TEST_RUN_RESULTS_FILE = path_from_root('out/previous_test_run_results.json')
 
-WEBIDL_BINDER = utils.bat_suffix(path_from_root('tools/webidl_binder'))
+WEBIDL_BINDER = exe_path_from_root('tools/webidl_binder')
 
-EMBUILDER = utils.bat_suffix(path_from_root('embuilder'))
-EMMAKE = utils.bat_suffix(path_from_root('emmake'))
-EMCMAKE = utils.bat_suffix(path_from_root('emcmake'))
-EMCONFIGURE = utils.bat_suffix(path_from_root('emconfigure'))
-EMRUN = utils.bat_suffix(utils.path_from_root('emrun'))
+EMBUILDER = exe_path_from_root('embuilder')
+EMMAKE = exe_path_from_root('emmake')
+EMCMAKE = exe_path_from_root('emcmake')
+EMCONFIGURE = exe_path_from_root('emconfigure')
+EMRUN = exe_path_from_root('emrun')
 WASM_DIS = os.path.join(building.get_binaryen_bin(), 'wasm-dis')
 LLVM_OBJDUMP = shared.llvm_tool_path('llvm-objdump')
 PYTHON = sys.executable
@@ -128,6 +128,10 @@ def copytree(src, dest):
   shutil.copytree(src, dest, dirs_exist_ok=True)
 
 
+def exe_suffix(cmd):
+  return cmd + '.exe' if WINDOWS else cmd
+
+
 def compiler_for(filename, force_c=False):
   if utils.suffix(filename) in ('.cc', '.cxx', '.cpp') and not force_c:
     return EMXX
@@ -249,7 +253,7 @@ def force_delete_dir(dirname):
   except PermissionError as e:
     # This issue currently occurs on Windows when running browser tests e.g.
     # on Firefox browser. Killing Firefox browser is not 100% watertight, and
-    # occassionally a Firefox browser process can be left behind, holding on
+    # occasionally a Firefox browser process can be left behind, holding on
     # to a file handle, preventing the deletion from succeeding.
     # We expect this issue to only occur on Windows.
     if not WINDOWS:
@@ -517,17 +521,15 @@ class RunnerCore(RetryableTestCase, metaclass=RunnerMeta):
     if self.is_browser_test():
       return
 
-    exp_args = ['--experimental-wasm-stack-switching', '--experimental-wasm-type-reflection']
     # Support for JSPI came earlier than 22, but the new API changes require v24
     if self.try_require_node_version(24):
-      self.node_args += exp_args
+      self.node_args += ['--experimental-wasm-stack-switching']
       return
 
     v8 = self.get_v8()
     if v8:
       self.cflags.append('-sENVIRONMENT=shell')
       self.js_engines = [v8]
-      self.v8_args += exp_args
       return
 
     self.fail('either d8 or node v24 required to run JSPI tests.  Use EMTEST_SKIP_JSPI to skip')
@@ -617,7 +619,7 @@ class RunnerCore(RetryableTestCase, metaclass=RunnerMeta):
     self.set_setting('NO_DEFAULT_TO_CXX')
     self.ldflags = []
     # Increase the stack trace limit to maximise usefulness of test failure reports.
-    # Also, include backtrace for all uncuaght exceptions (not just Error).
+    # Also, include backtrace for all uncaught exceptions (not just Error).
     self.node_args = ['--stack-trace-limit=50', '--trace-uncaught']
     self.spidermonkey_args = ['-w']
 
@@ -697,31 +699,31 @@ class RunnerCore(RetryableTestCase, metaclass=RunnerMeta):
       os.chdir(os.path.dirname(self.get_dir()))
       force_delete_dir(self.get_dir())
 
-      if EMTEST_DETECT_TEMPFILE_LEAKS and not DEBUG:
-        temp_files_after_run = []
-        for root, dirnames, filenames in os.walk(self.temp_dir):
-          for dirname in dirnames:
-            temp_files_after_run.append(os.path.normpath(os.path.join(root, dirname)))
-          for filename in filenames:
-            temp_files_after_run.append(os.path.normpath(os.path.join(root, filename)))
+    if EMTEST_DETECT_TEMPFILE_LEAKS:
+      temp_files_after_run = []
+      for root, dirnames, filenames in os.walk(self.temp_dir):
+        for dirname in dirnames:
+          temp_files_after_run.append(os.path.normpath(os.path.join(root, dirname)))
+        for filename in filenames:
+          temp_files_after_run.append(os.path.normpath(os.path.join(root, filename)))
 
-        # Our leak detection will pick up *any* new temp files in the temp dir.
-        # They may not be due to us, but e.g. the browser when running browser
-        # tests. Until we figure out a proper solution, ignore some temp file
-        # names that we see on our CI infrastructure.
-        ignorable_file_prefixes = [
-          '/tmp/tmpaddon',
-          '/tmp/circleci-no-output-timeout',
-          '/tmp/wasmer',
-        ]
+      # Our leak detection will pick up *any* new temp files in the temp dir.
+      # They may not be due to us, but e.g. the browser when running browser
+      # tests. Until we figure out a proper solution, ignore some temp file
+      # names that we see on our CI infrastructure.
+      ignorable_file_prefixes = [
+        '/tmp/tmpaddon',
+        '/tmp/circleci-no-output-timeout',
+        '/tmp/wasmer',
+      ]
 
-        left_over_files = set(temp_files_after_run) - set(self.temp_files_before_run)
-        left_over_files = [f for f in left_over_files if not any(f.startswith(p) for p in ignorable_file_prefixes)]
-        if left_over_files:
-          errlog(f'ERROR: After running test, there are {len(left_over_files)} new temporary files/directories left behind:')
-          for f in left_over_files:
-            errlog('leaked file: ', f)
-          self.fail(f'Test leaked {len(left_over_files)} temporary files!')
+      left_over_files = set(temp_files_after_run) - set(self.temp_files_before_run)
+      left_over_files = [f for f in left_over_files if not any(f.startswith(p) for p in ignorable_file_prefixes)]
+      if left_over_files:
+        errlog(f'ERROR: After running test, there are {len(left_over_files)} new temporary files/directories left behind:')
+        for f in left_over_files:
+          errlog('leaked file: ', f)
+        self.fail(f'Test leaked {len(left_over_files)} temporary files!')
 
   def get_setting(self, key, default=None):
     return self.settings_mods.get(key, default)
@@ -881,7 +883,7 @@ class RunnerCore(RetryableTestCase, metaclass=RunnerMeta):
     return len(non_data_lines)
 
   def clean_js_output(self, output):
-    """Cleaup the JS output prior to running verification steps on it.
+    """Cleanup the JS output prior to running verification steps on it.
 
     Due to minification, when we get a crash report from JS it can sometimes
     contains the entire program in the output (since the entire program is
@@ -1091,7 +1093,7 @@ class RunnerCore(RetryableTestCase, metaclass=RunnerMeta):
     self.assertEqual(read_binary(file1),
                      read_binary(file2))
 
-  library_cache: Dict[str, Tuple[str, object]] = {}
+  library_cache: dict[str, tuple[str, object]] = {}
 
   def get_build_dir(self):
     ret = self.in_dir('building')
@@ -1158,7 +1160,7 @@ class RunnerCore(RetryableTestCase, metaclass=RunnerMeta):
     # core emscripten code (shared.py) here.
 
     # Handle buffering for subprocesses.  The python unittest buffering mechanism
-    # will only buffer output from the current process (by overwriding sys.stdout
+    # will only buffer output from the current process (by overriding sys.stdout
     # and sys.stderr), not from sub-processes.
     stdout_buffering = 'stdout' not in kwargs and isinstance(sys.stdout, io.StringIO)
     stderr_buffering = 'stderr' not in kwargs and isinstance(sys.stderr, io.StringIO)
@@ -1214,7 +1216,7 @@ class RunnerCore(RetryableTestCase, metaclass=RunnerMeta):
     self.assertContained(expected, err)
     return err
 
-  # excercise dynamic linker.
+  # exercise dynamic linker.
   #
   # test that linking to shared library B, which is linked to A, loads A as well.
   # main is also linked to C, which is also linked to A. A is loaded/initialized only once.
@@ -1224,7 +1226,7 @@ class RunnerCore(RetryableTestCase, metaclass=RunnerMeta):
   #          C
   #
   # this test is used by both test_core and test_browser.
-  # when run under browser it excercises how dynamic linker handles concurrency
+  # when run under browser it exercises how dynamic linker handles concurrency
   # - because B and C are loaded in parallel.
   def _test_dylink_dso_needed(self, do_run):
     create_file('liba.cpp', r'''
@@ -1477,7 +1479,7 @@ class RunnerCore(RetryableTestCase, metaclass=RunnerMeta):
       self.cflags += cflags
     # inflate.c does -1L << 16
     self.cflags.append('-Wno-shift-negative-value')
-    # adler32.c uses K&R sytyle function declarations
+    # adler32.c uses K&R style function declarations
     self.cflags.append('-Wno-deprecated-non-prototype')
     # Work around configure-script error. TODO: remove when
     # https://github.com/emscripten-core/emscripten/issues/16908 is fixed
