@@ -1,5 +1,5 @@
-// Example TS program that consumes the emscripten-generated module to to
-// illustrate how the type definitions are used and test they are workings as
+// Example TS program that consumes the emscripten-generated module to
+// illustrate how the type definitions are used and test they are working as
 // expected.
 
 // The imported file will either be an ES module or a CommonJS module depending
@@ -12,19 +12,33 @@ import moduleFactory from './embind_tsgen.js';
 
   const module = await moduleFactory();
 
-  // Test a few variations of passing value_objects with strings.
   module.setValObj({
-    bar: module.Bar.valueOne,
+    firstEnum: module.FirstEnum.kValueOne,
+    secondEnum: module.SecondEnum.kValueA,
+    thirdEnum: module.ThirdEnum.kValueAlpha,
     string: "ABCD",
     callback: () => {}
   });
 
+  // Test a few variations of passing enum value
   module.setValObj({
-    bar: module.Bar.valueOne,
+    firstEnum: module.FirstEnum.kValueOne,
+    secondEnum: 0,
+    thirdEnum: "kValueAlpha",
     string: new Int8Array([65, 66, 67, 68]),
     callback: () => {}
   });
 
+  // Test optional field
+  module.setValObj({
+    firstEnum: module.FirstEnum.kValueOne,
+    secondEnum: module.SecondEnum.kValueA,
+    thirdEnum: module.ThirdEnum.kValueAlpha,
+    string: "ABCD",
+    callback: () => {},
+    optionalInt: 99
+  });
+
   const valObj = module.getValObj();
   // TODO: remove the cast below when better definitions are generated for value
   // objects.
