@@ -344,7 +344,7 @@ double prevT = 0;
 
 void register_tasks()
 {
-    numTasks = EM_ASM_INT(return (typeof document !== 'undefined' && document.getElementById('num_threads')) ? parseInt(document.getElementById('num_threads').value) : 1);
+    numTasks = EM_ASM_INT(return globalThis.document?.getElementById('num_threads') ? parseInt(document.getElementById('num_threads').value) : 1);
 
 #ifdef SINGLETHREADED
   // Single-threaded
@@ -363,7 +363,7 @@ void register_tasks()
 #else
   emscripten_atomic_fence();
 
-  numTasks = EM_ASM_INT(return (typeof document !== 'undefined' && document.getElementById('num_threads')) ? parseInt(document.getElementById('num_threads').value) : 1);
+  numTasks = EM_ASM_INT(return globalThis.document?.getElementById('num_threads') ? parseInt(document.getElementById('num_threads').value) : 1);
   if (numTasks < 1) numTasks = 1;
   if (numTasks > emscripten_num_logical_cores()) numTasks = emscripten_num_logical_cores();
 
@@ -481,7 +481,7 @@ void main_tick()
   }
 #endif
 
-  int new_use_sse = EM_ASM_INT(return (typeof document !== 'undefined' && document.getElementById('use_sse')) ? document.getElementById('use_sse').checked : false);
+  int new_use_sse = EM_ASM_INT(return globalThis.document?.getElementById('use_sse') ? document.getElementById('use_sse').checked : false);
 
   if (numItersDoneOnCanvas >= minItersBeforeDisplaying || new_use_sse != use_sse)
   {
@@ -498,12 +498,12 @@ void main_tick()
   use_sse = new_use_sse;
 
   numItersPerFrame = EM_ASM_INT({
-    if (typeof location !== 'undefined') {
+    if (globalThis.location) {
       var updatesPerFrame = (new RegExp("[\\?&]updates=([^&#]*)")).exec(location.href);
       if (updatesPerFrame) return updatesPerFrame[1];
     }
     if (arguments_ && arguments_.length >= 1) return parseInt(arguments_[0]);
-    if (typeof document !== 'undefined' && document.getElementById('updates_per_frame')) return parseInt(document.getElementById('updates_per_frame').value);
+    if (globalThis.document?.getElementById('updates_per_frame')) return parseInt(document.getElementById('updates_per_frame').value);
     return 50;
   });
   if (numItersPerFrame < 10) numItersPerFrame = 10;
