@@ -10,9 +10,15 @@ import re
 import shlex
 
 import common
-from common import RunnerCore
-from common import parameterized, node_pthreads
-from common import test_file, read_file, read_binary, create_file, compiler_for
+from common import (
+  RunnerCore,
+  compiler_for,
+  create_file,
+  read_binary,
+  read_file,
+  test_file,
+)
+from decorators import node_pthreads, parameterized
 
 from tools import building, shared
 
@@ -36,6 +42,7 @@ class codesize(RunnerCore):
     'random_printf_wasm2js': ('random_printf', True),
     'hello_webgl_wasm': ('hello_webgl', False),
     'hello_webgl_wasm2js': ('hello_webgl', True),
+    'hello_webgl2_wasm_singlefile': ('hello_webgl2_wasm_singlefile', False),
     'hello_webgl2_wasm': ('hello_webgl2', False),
     'hello_webgl2_wasm2js': ('hello_webgl2', True),
     'math': ('math', False),
@@ -80,6 +87,7 @@ class codesize(RunnerCore):
                            '-lGL',
                            '-sMODULARIZE']
     hello_webgl2_sources = hello_webgl_sources + ['-sMAX_WEBGL_VERSION=2']
+    hello_webgl2_wasm_singlefile_sources = hello_webgl2_sources + ['-sSINGLE_FILE']
     hello_wasm_worker_sources = [test_file('wasm_worker/wasm_worker_code_size.c'), '-sWASM_WORKERS', '-sENVIRONMENT=web']
     audio_worklet_sources = [test_file('webaudio/audioworklet.c'), '-sWASM_WORKERS', '-sAUDIO_WORKLET', '-sENVIRONMENT=web', '-sTEXTDECODER=1']
     embind_hello_sources = [test_file('codesize/embind_hello_world.cpp'), '-lembind']
@@ -91,6 +99,7 @@ class codesize(RunnerCore):
       'hello_webgl': hello_webgl_sources,
       'math': math_sources,
       'hello_webgl2': hello_webgl2_sources,
+      'hello_webgl2_wasm_singlefile': hello_webgl2_wasm_singlefile_sources,
       'hello_wasm_worker': hello_wasm_worker_sources,
       'audio_worklet': audio_worklet_sources,
       'embind_val': embind_val_sources,
@@ -243,7 +252,7 @@ class codesize(RunnerCore):
     relevant = relevant.split(',')
     sent = [x.split(':')[0].strip() for x in relevant]
     sent = [x for x in sent if x]
-    # Deminify the sent list, if minification occured
+    # Deminify the sent list, if minification occurred
     if os.path.exists('minify.map'):
       sent = deminify_syms(sent, 'minify.map')
       os.remove('minify.map')
@@ -260,7 +269,7 @@ class codesize(RunnerCore):
       outputs.append('a.out.nodebug.wasm')
 
       imports, exports, funcs = self.parse_wasm('a.out.wasm')
-      # Deminify the imports/export lists, if minification occured
+      # Deminify the imports/export lists, if minification occurred
       if os.path.exists('minify.map'):
         exports = deminify_syms(exports, 'minify.map')
         imports = [i.split('.', 1)[1] for i in imports]
@@ -350,7 +359,7 @@ class codesize(RunnerCore):
     # finally, check what happens when we export nothing. wasm should be almost empty
     'export_nothing': (['-Os', '-sEXPORTED_FUNCTIONS=[]'],),
     # we don't metadce with linkable code! other modules may want stuff
-    # TODO(sbc): Investivate why the number of exports is order of magnitude
+    # TODO(sbc): Investigate why the number of exports is order of magnitude
     # larger for wasm backend.
     # This test seems to produce different results under gzip on macOS and Windows machines
     # so skip the gzip size reporting here.
@@ -404,3 +413,13 @@ class codesize(RunnerCore):
   def test_small_js_flags(self):
     self.emcc('browser_test_hello_world.c', ['-O3', '--closure=1', '-sINCOMING_MODULE_JS_API=[]', '-sENVIRONMENT=web', '--output-eol=linux'])
     self.check_output_sizes('a.out.js')
+
+  # This test verifies that gzipped binary-encoded a SINGLE_FILE build results in a smaller size
+  # than gzipped base64-encoded version.
+  def test_binary_encode_is_smaller_than_base64_encode(self):
+    self.emcc('hello_world.c', ['-O2', '-sSINGLE_FILE', '-sSINGLE_FILE_BINARY_ENCODE'])
+    size_binary_encode = len(gzip.compress(read_binary('a.out.js')))
+    self.emcc('hello_world.c', ['-O2', '-sSINGLE_FILE', '-sSINGLE_FILE_BINARY_ENCODE=0'])
+    size_base64 = len(gzip.compress(read_binary('a.out.js')))
+    print(f'Binary encoded file size: {size_binary_encode}, base64 encoded file size: {size_base64}')
+    self.assertLess(size_binary_encode, size_base64)
